;; init.asm - Extended boot loader - System Initialization
;;
;; Copyright (C) 1999 Thomas Novelli
;;	Retro is distributed under the terms of the GNU LGPL, version 2 or
;;	greater. (For the text of the license, see the file COPYING.LIB)
;;

%define delay	jmp short $+2

%define BOOTSEG		0x7C0	; BIOS puts the boot sector at address 7C00

BITS 16

init:
%if 0
dump_shit:
	mov si,0x7E00
	mov cx,0x800
	mov ah,0x0e
	xor bx,bx
.19	lodsb
	cmp al,' '
	jb .skip
	cmp al,127
	jae .skip
	int 0x10
.skip	loop .19
	hang
%endif
;;---------------------------- Tests & Checks -----------------------------

;; Get extended memory size (kB)
	mov	ah, 0x88
	int	0x15
	mov	[0x4000], ax	;Store the result somewhere for later
				;0x4000 is temporary..

;; Set keyboard repeat rate to 'fast'
	mov	ax, 0x0305
	xor	bx, bx
	int	0x16

;; Integrity check? (CRC? Checksum? Signature at end?)


;; Check for proper CPU type & mode
	call	check_cpu_type		;Must be 386 or better
	call	check_mode		;Must be in real mode

;; Check for FPU?


;; Get ready for PMODE
	call	build_GDT
	call	build_IDT
	call	reprogram_PICs
	call	enable_a20

;; ### Initialize PMODE ###
	cli
	lgdt	[GDT_ADDRESS]
	lidt	[idtr]
	mov	eax, CR0
	inc	eax
	mov	CR0, eax
	jmp	dword 8:pmode


;-------------------------------------------------------------------------


;; Enable the A20 line so we can access memory above 1M
enable_a20:
	call	empty_8042
;.5:	in	al, 0x64
;	test	al, 2
;	jnz	.5
	mov	al, 0xD1	;write command
	out	0x64, al
	call	empty_8042
;.6	in	al, 0x64
;	test	al, 2
;	jnz	.6
	mov	al, 0xDF	;Enable A20
	out	0x60, al
	call	empty_8042
;; Wait for A20 to actually turn on (on some computers, especially laptops,
;; it takes long enough to cause problems)
%define TEST_ADDR	0x7c
	push	ds
	xor	ax, ax		;ds = 0x0000
	mov	ds, ax
	dec	ax		;gs = 0xffff
	mov	gs, ax
	mov	bx, [TEST_ADDR]	;save value
.wait	inc	ax		;ax = 0
	mov	[TEST_ADDR], ax
	cmp	ax, [gs:TEST_ADDR+0x10]	;test wrapped-around location
	je	.wait			;wait until wrap-around stops
	mov	[TEST_ADDR], bx	;restore value
	pop	ds
	ret


;Flush the keyboard controller
empty_8042
	delay
        in      al, 0x64        ;Status port
        test    al, 1           ;Output buffer?
        jz      .no_output
	delay
        in      al, 0x60        ;Read byte
        jmp short empty_8042
.no_output
        test    al, 2           ;Input buffer full?
        jnz     empty_8042      ;Yes? repeat...
        ret


;Reprogram 8259A programmable interrupt controllers
;
;The BIOS puts interrupts at 0x08-0x0F, and we want them at 0x20-0x2F,
;right after the reserved interrupts, where they belong.
;
;Code snipped from John Fine's V86TEST
;
reprogram_PICs:
	M_PIC	EQU	0x20		;I/O for master PIC
	M_IMR	EQU	0x21		;I/O for master IMR
	S_PIC	EQU	0xA0		;I/O for slave PIC
	S_IMR	EQU	0xA1		;I/O for slace IMR

	EOI	EQU	0x20		;EOI command

	ICW1	EQU	0x11		;Cascade, Edge triggered
					;ICW2 is vector
					;ICW3 is slave bitmap or number
	ICW4	EQU	0x01		;8088 mode

	M_VEC	EQU	0x20		;Vector for master
	S_VEC	EQU	0x28		;Vector for slave

	OCW3_IRR EQU	0x0A		;Read IRR
	OCW3_ISR EQU	0x0B		;Read ISR

	mov	al, ICW1		;Start 8259 initialization
	out	M_PIC, al
	out	S_PIC, al
	mov	al, M_VEC		;Base interrupt vector
	out	M_PIC+1, al
	mov	al, S_VEC
	out	S_PIC+1, al
	mov	al, 1<<2		;Bitmask for cascade on IRQ 2
	out	M_PIC+1, al
	mov	al, 2			;Cascade on IRQ 2
	out	S_PIC+1, al
	mov	al, ICW4		;Finish 8259 initialization
	out	M_PIC+1, al
	out	S_PIC+1, al

	mov	al, 0xFF		;Mask all interrupts
	out	M_IMR, al
	out	S_IMR, al

	ret



%if 0	;Not using this code now..
;"Real mode switch routine" from Linux (setup.S), called before pmode transition
default_switch
        cli
        mov     al, 0x80        ;Disable NMI for bootup sequence
        out     0x70, al	;Note: port 0x70 = RTC
        retf
%endif



;Check for 386, etc.
check_cpu_type:
	pushf
	xor	ah, ah
	push	ax
	popf
	pushf
	pop	ax
	and	ah, 0xF0
	cmp	ah, 0xF0
	je	.no386
	mov	ah, 0x70
	push	ax
	popf
	pushf
	pop	ax
	and	ah, 0x70
	jz	.no386
	popf
	ret
.no386
	mov	bp, .msg
	call	print_string
	jmp short $
.msg	db	"You need at least a 386 to run in protected mode!",10,13,0



;Make sure we're starting from Real mode
check_mode:
	mov	eax, CR0
	and	al, 1
	jnz	.not_real_mode
	ret
.not_real_mode
	mov	bp, .msg
	call	print_string
	jmp short $
.msg	db	"You're already in protected mode! WTF?",10,13,0


;-------------------------------------------------------------------------


%include "gdt.inc"

	IDT_ADDRESS	EQU	0x600
build_IDT:
;
; Build a full-size IDT (256 entries), with all of them handled by the
; "Unknown Interrupt" routine. Interrupt handlers will be expected to
; install themselves by modifying the IDT.
;
	mov	ax, (IDT_ADDRESS >> 4)	;ES:DI = start of IDT
	mov	es, ax
	xor	di, di
	mov	cx, 256			;CX = # of entries
	cld
.loop
	push	cx

	mov	si, .entry		;DS:SI = template entry
	mov	cx, 4
	rep	movsw			;Copy it...

	pop	cx
	loop	.loop
	ret
;
.entry	idesc	flat_code, default_int_handler, D_INT



idtr:	;For the IDT generated on the fly (by build_IDT)
	dw	256*8
	dd	IDT_ADDRESS



	GDT_ADDRESS	equ	0xE00
build_GDT:
	mov	ax, (GDT_ADDRESS >> 4)
	mov	es, ax
	xor	di, di
	mov	si, gdt
	mov	cx, (gdt_end - gdt) / 2
	cld
	rep	movsw

	xor	di, di				; set GDTR base
	mov	[es:di+2], word GDT_ADDRESS
	mov	[es:di+4], word 0
	ret

ALIGN 4

;General Descriptor Table
gdt:
	start_gdt	; gdtr
flat_code	desc	0, 0xFFFFF, D_CODE+D_READ+D_BIG+D_BIG_LIM
flat_data	desc	0, 0xFFFFF, D_DATA+D_WRITE+D_BIG+D_BIG_LIM
user_code	desc	0, 0xFFFFF, D_CODE+D_READ+D_BIG+D_BIG_LIM+D_DPL3
user_data	desc	0, 0xFFFFF, D_DATA+D_WRITE+D_BIG+D_BIG_LIM+D_DPL3
tss_sel		desc	0, 0, D_TSS
	end_gdt
gdt_end


%include "pmode.inc"
