; Boot sector to load up an OS kernel
;
; Copyright (C) 1999 Thomas Novelli
;	Retro is distributed under the terms of the GNU LGPL, version 2 or
;	greater. (For the text of the license, see the file COPYING.LIB)
;
; Platform: Intel x86
;	Thanks for the tips & examples: Linus Torvalds, John S. Fine,
;	M. J. Vines, Francois-Rene Rideau
;
; To do:
;	Eventually, load kernel from a file/object (after the persistent
;	storage system has stabilized)
;
;
; Here's how it works:
;
; BIOS: 
;  1. Loads Sector 1, Track 0, Head 0 to address 7C00
;  2. Checks the last two bytes of the bootsector, which must contain
;     the signature 0xAA55, or it says "Operating system not found".
;  3. Loads DL with:
;	0x00 if booted from drive A
;	0x80 if booted from drive C
;	(Note: What about bootable CD's? 0x81, 0x82, etc?)
;  4. Jumps to 0000:7C00
;
; Boot loader:
;  1. Initializes segments & stack
;  2. Prints a message
;  3. Loads the rest of the boot loader, starting at address 7E00
;
; Initialization:
;  1. Checks some things, like the CPU type, RAM availble, etc.
;  2. Starts protected mode (386+ only)
;
; Kernel:
;  The kernel (core of the operating system) is responsible for loading
;  and managing all programs/objects.

; Memory map:
;	0000 to 03FF	; Real mode Interrupt Vector Table
;	0400 to 049F	; BIOS data area
;	7000 to 7BFF	; Stack for boot loader
;	7C00 to 7DFF	; Boot sector
;	7E00 to ????	; Extended boot loader & OS kernel

BITS 16

;Note: we could detect the boot address if that helps. CALL $+2 and POP..

%define BOOTSEG		0x7C0	; BIOS puts the boot sector at address 7C00
%define STACKSEG	0x700	; This area should be free for the stack...

%macro hexp 1	;; hexp <number>
	pusha
	push word %1
	mov	bp, sp
	call	hex_print
	call	print_nl
	popa
%endmacro

%define hang	jmp short $


	ORG	0x7E00
;mov ax, 0xE00+'*'
;xor bx, bx
;int 0x10

jmp short init


print_string:
;
; Let ES:BP point to a null-terminated string
;
        mov     ah,0Eh
        mov     bx,0            ;Set video page & Text olor
.l1     cmp     [ds:bp],byte 0
        je      .end
        mov     al,[ds:bp]
        int     10h
        inc     bp
        jmp     .l1
.end    ret

%include "init.inc"
